<!--  -->
<template>
  <div style="width: 100%; height: calc(100% - 40px)">
    <div style="height: 40px">
      <a-button class="operationbtn" @click="exportJson" type="primary"> 保存 </a-button>
      <a-button class="operationbtn" @click="undo" type="primary" title="撤销当前操作(ctrl+z)"
        >撤销
      </a-button>
      <a-button class="operationbtn" @click="redo" type="primary" title="重做(ctrl+shift+z)"
        >重做
      </a-button>
    </div>

    <div id="container" style="width: 100%; height: 100%; display: flex">
      <div id="stencil" style="width: 200px"></div>
      <div id="graph-container" style="flex: 1"></div>
      <div
        :style="`width:300px;border:1px solid #e2e2e2;padding:5px;`"
        v-show="rightBoxVisible"
      >
        <div
          style="
            display: flex;
            background-color: #1890ff;
            color: #fff;
            height: 30px;
            line-height: 30px;
          "
        >
          <div style="flex: 1; text-align: center">样式</div>
          <div
            @click="onClose"
            style="
              width: 30px;
              height: 30px;
              line-height: 30px;
              text-align: center;
              background-color: transparent;
              cursor: pointer;
            "
          >
            <a-icon type="close" />
          </div>
        </div>
        <a-form
          v-show="lineForm"
          style="overflow: auto; height: calc(100% - 30px)"
          layout="horizontal"
          :label-col="formItemLayout.labelCol"
          :wrapper-col="formItemLayout.wrapperCol"
        >
          <a-form-item label="开启动画">
            <a-switch
              checked-children="开"
              un-checked-children="关"
              v-model="line.animation"
              :disabled="line.type != 'dashed'"
              @change="lineAnimationChange"
            />
          </a-form-item>
          <a-form-item label="线宽">
            <a-input
              v-model="line.width"
              @change="lineWidthChange"
              type="number"
              min="0"
              suffix="px"
              placeholder="线宽"
            />
          </a-form-item>
          <a-form-item label="线形">
            <a-select
              :default-value="line.type"
              @change="lineTypeChange"
              placeholder="选择线形"
            >
              <a-select-option value="solid"> —————— </a-select-option>
              <a-select-option value="dashed">
                - - - - - - - - - -
              </a-select-option>
            </a-select>
          </a-form-item>
          <a-form-item label="线颜色">
            <el-color-picker
              v-model="line.stroke"
              v-on:change="lineColorChange"
              size="mini"
            ></el-color-picker>
          </a-form-item>
        </a-form>
      </div>
    </div>
  </div>
</template>

<script>
import { Graph, Shape } from "@antv/x6";

import { register } from "@antv/x6-vue-shape";

import { Stencil } from "@antv/x6-plugin-stencil";
import { Transform } from "@antv/x6-plugin-transform";
import { Selection } from "@antv/x6-plugin-selection";
import { Snapline } from "@antv/x6-plugin-snapline";
import { Keyboard } from "@antv/x6-plugin-keyboard";
import { Clipboard } from "@antv/x6-plugin-clipboard";
import { History } from "@antv/x6-plugin-history";
import insertCss from "insert-css";
import img102 from "@/assets/images/102.png";
import imgavatar from "@/assets/images/avatar2.jpg";

function preWork() {
  // 这里协助演示的代码，在实际项目中根据实际情况进行调整
  // const container = document.getElementById("container");
  // const stencilContainer = document.createElement("div");
  // stencilContainer.id = "stencil";
  // const graphContainer = document.createElement("div");
  // graphContainer.id = "graph-container";
  // container.appendChild(stencilContainer);
  // container.appendChild(graphContainer);

  insertCss(`
            #container {
            display: flex;
            border: 1px solid #dfe3e8;
            }
            #stencil {
            width: 200px;
            height: 100%;
            position: relative;
            border-right: 1px solid #dfe3e8;
            }
            #graph-container {
            width: calc(100% - 201px)!important;
            height: 100%!important;
            }
            .x6-widget-stencil  {
            background-color: #fff;
            }
            .x6-widget-stencil-title {
            background-color: #fff;
            }
            .x6-widget-stencil-group-title {
            background-color: #fff !important;
            }
            .x6-widget-transform {
            margin: -1px 0 0 -1px;
            padding: 0px;
            border: 1px solid #239edd;
            }
            .x6-widget-transform > div {
            border: 1px solid #239edd;
            }
            .x6-widget-transform > div:hover {
            background-color: #3dafe4;
            }
            .x6-widget-transform-active-handle {
            background-color: #3dafe4;
            }
            .x6-widget-transform-resize {
            border-radius: 0;
            }
            .x6-widget-selection-inner {
            border: 1px solid #239edd;
            }
            .x6-widget-selection-box {
            opacity: 0;
            }
        `);
}

let m_graph = null;
export default {
  name: "",
  components: {},
  data() {
    return {
      /**样式操作 */
      rightBoxVisible: false,
      /**线修改form */
      lineForm: false,
      formItemLayout: {
        labelCol: { span: 8 },
        wrapperCol: { span: 14 },
      },

      /**是否可以重做 */
      isredo: false,
      /**是否可以撤销 */
      isundo: false,
      /**线的样式 */
      line: {
        width: 2,
        strokeDasharray: 5,
        stroke: "#A2B1C3",
        type: "solid",
        animation: false,
      },
      /**选中的元素 */
      selectedCell: null,
    };
  },
  created() {},
  mounted() {
    preWork();
    const graph = (m_graph = new Graph({
      container: document.getElementById("graph-container"),
      grid: true,
      mousewheel: {
        enabled: true,
        zoomAtMousePosition: true,
        modifiers: "ctrl",
        minScale: 0.5,
        maxScale: 3,
      },
      connecting: {
        router: {
          name: "manhattan",
          args: {
            padding: 10,
          },
        },
        connector: {
          name: "rounded",
          args: {
            radius: 5,
          },
        },
        anchor: "center",
        edgeAnchor: "orth",
        connectionPoint: "anchor",
        allowBlank: false,
        snap: {
          radius: 30,
        },

        createEdge() {
          return new Shape.Edge({
            attrs: {
              line: {
                // stroke: "#A2B1C3",
                stroke: "red",
                strokeWidth: 2,
                strokeDasharray: null,
                targetMarker: {
                  name: "block",
                  width: 12,
                  height: 8,
                },
              },
            },
            zIndex: 0,
          });
        },
        validateConnection({ targetMagnet }) {
          return !!targetMagnet;
        },
      },
      interacting: function (cellView) {
        if (cellView.cell.getProp("customLinkInteractions")) {
          return { vertexAdd: false };
        }
        return true;
      },
      highlighting: {
        magnetAdsorbed: {
          name: "stroke",
          args: {
            attrs: {
              fill: "#5F95FF",
              stroke: "#5F95FF",
            },
          },
        },
      },
    }));

    graph.on("edge:mouseenter", ({ cell }) => {
      // cell.addTools("vertices", "onhover");
      // cell.addTools("vertices", );

      cell.addTools([
        {
          name: "target-arrowhead",
          args: {
            attrs: {
              fill: "red",
            },
          },
        },
        // {
        //   name: "button-remove",
        // },
      ]);
    });

    graph.on("edge:mouseleave", ({ cell }) => {
      if (cell.hasTools("onhover")) {
      }
      cell.removeTools();
    });

    graph.on("cell:dblclick", ({ cell, e }) => {
      const isNode = cell.isNode();
      const name = cell.isNode() ? "node-editor" : "edge-editor";
      cell.removeTool(name);
      cell.addTools({
        name,
        args: {
          event: e,
          attrs: {
            backgroundColor: isNode ? "#EFF4FF" : "#FFF",
          },
        },
      });
    });

    graph.on("edge:click", (val) => {
      let { cell, e } = val;
      const atts = cell.getAttrs();
      this.showDrawer();
      this.selectedCell = cell;
     this.line.stroke = atts.line.stroke
     this.line.width = atts.line.strokeWidth
     this.line.type = atts.line.strokeDasharray?"dashed":"solid"
     if(atts.line.strokeDasharray&&atts.line.style&&atts.line.style.animation){
      this.line.animation = true
     }
     this.lineForm = true
     
      console.log(cell, val, atts, 216);
    });

    // #region 使用插件
    graph
      .use(
        new Transform({
          resizing: true,
          rotating: true,
        })
      )
      .use(
        new Selection({
          enabled: true,
          rubberband: true,
          showNodeSelectionBox: true,
        })
      )
      .use(
        new Snapline({
          enabled: true,
        })
      )
      .use(
        new Keyboard({
          enabled: true,
        })
      )
      .use(
        new Clipboard({
          enabled: true,
        })
      )
      .use(
        new History({
          enabled: true,
        })
      );
    // #endregion

    // #region 初始化 stencil
    const stencil = new Stencil({
      title: "流程图",
      target: graph,
      stencilGraphWidth: 200,
      stencilGraphHeight: 250,
      collapsable: true,
      groups: [
        {
          title: "基础流程图",
          name: "group1",
        },
        {
          title: "系统设计图",
          name: "group2",
          graphHeight: 250,
          layoutOptions: {
            rowHeight: 70,
          },
        },
      ],
      layoutOptions: {
        columns: 2,
        columnWidth: 80,
        rowHeight: 55,
      },
    });
    document.getElementById("stencil").appendChild(stencil.container);
    // #endregion
console.log(graph,4000)
    // #region 快捷键与事件
    graph.bindKey(["meta+c", "ctrl+c"], () => {
      const cells = graph.getSelectedCells();
      if (cells.length) {
        graph.copy(cells);
      }
      return false;
    });
    graph.bindKey(["meta+x", "ctrl+x"], () => {
      const cells = graph.getSelectedCells();
      if (cells.length) {
        graph.cut(cells);
      }
      return false;
    });
    graph.bindKey(["meta+v", "ctrl+v"], () => {
      if (!graph.isClipboardEmpty()) {
        const cells = graph.paste({ offset: 32 });
        graph.cleanSelection();
        graph.select(cells);
      }
      return false;
    });

    //undo redo
    graph.bindKey(["meta+z", "ctrl+z"], () => {
      if (graph.canUndo()) {
        graph.undo();
      }
      return false;
    });
    graph.bindKey(["meta+shift+z", "ctrl+shift+z"], () => {
      if (graph.canRedo()) {
        graph.redo();
      }
      return false;
    });

    // select all
    graph.bindKey(["meta+a", "ctrl+a"], () => {
      const nodes = graph.getNodes();
      if (nodes) {
        graph.select(nodes);
      }
    });

    //delete
    graph.bindKey("backspace", () => {
      const cells = graph.getSelectedCells();
      if (cells.length) {
        this.selectedCell = null
        graph.removeCells(cells);
      }
    });

    // zoom
    graph.bindKey(["ctrl+1", "meta+1"], () => {
      const zoom = graph.zoom();
      if (zoom < 1.5) {
        graph.zoom(0.1);
      }
    });
    graph.bindKey(["ctrl+2", "meta+2"], () => {
      const zoom = graph.zoom();
      if (zoom > 0.5) {
        graph.zoom(-0.1);
      }
    });

    // 控制连接桩显示/隐藏
    const showPorts = (ports, show) => {
      for (let i = 0, len = ports.length; i < len; i = i + 1) {
        ports[i].style.visibility = show ? "visible" : "hidden";
      }
    };
    graph.on("node:mouseenter", () => {
      const container = document.getElementById("graph-container");
      const ports = container.querySelectorAll(".x6-port-body");
      showPorts(ports, true);
    });
    graph.on("node:mouseleave", () => {
      const container = document.getElementById("graph-container");
      const ports = container.querySelectorAll(".x6-port-body");
      showPorts(ports, false);
    });
    // #endregion

    // #region 初始化图形
    const ports = {
      groups: {
        top: {
          position: "top",
          attrs: {
            circle: {
              r: 4,
              magnet: true,
              stroke: "#5F95FF",
              strokeWidth: 1,
              fill: "#fff",
              style: {
                visibility: "hidden",
              },
            },
          },
        },
        right: {
          position: "right",
          attrs: {
            circle: {
              r: 4,
              magnet: true,
              stroke: "#5F95FF",
              strokeWidth: 1,
              fill: "#fff",
              style: {
                visibility: "hidden",
              },
            },
          },
        },
        bottom: {
          position: "bottom",
          attrs: {
            circle: {
              r: 4,
              magnet: true,
              stroke: "#5F95FF",
              strokeWidth: 1,
              fill: "#fff",
              style: {
                visibility: "hidden",
              },
            },
          },
        },
        left: {
          position: "left",
          attrs: {
            circle: {
              r: 4,
              magnet: true,
              stroke: "#5F95FF",
              strokeWidth: 1,
              fill: "#fff",
              style: {
                visibility: "hidden",
              },
            },
          },
        },
      },
      items: [
        {
          group: "top",
        },
        {
          group: "right",
        },
        {
          group: "bottom",
        },
        {
          group: "left",
        },
      ],
    };

    Graph.registerNode(
      "custom-rect",
      {
        inherit: "rect",
        width: 66,
        height: 36,
        attrs: {
          body: {
            strokeWidth: 1,
            stroke: "#5F95FF",
            fill: "#EFF4FF",
          },
          text: {
            fontSize: 12,
            fill: "#262626",
          },
        },
        ports: { ...ports },
      },
      true
    );

    Graph.registerNode(
      "custom-polygon",
      {
        inherit: "polygon",
        width: 66,
        height: 36,
        attrs: {
          body: {
            strokeWidth: 1,
            stroke: "#5F95FF",
            fill: "#EFF4FF",
          },
          text: {
            fontSize: 12,
            fill: "#262626",
          },
        },
        ports: {
          ...ports,
          items: [
            {
              group: "top",
            },
            {
              group: "bottom",
            },
          ],
        },
      },
      true
    );

    Graph.registerNode(
      "custom-circle",
      {
        inherit: "circle",
        width: 45,
        height: 45,
        attrs: {
          body: {
            strokeWidth: 1,
            stroke: "#5F95FF",
            fill: "#EFF4FF",
          },
          text: {
            fontSize: 12,
            fill: "#262626",
          },
        },
        ports: { ...ports },
      },
      true
    );

    Graph.registerNode(
      "custom-image",
      {
        inherit: "rect",
        width: 52,
        height: 52,
        markup: [
          {
            tagName: "rect",
            selector: "body",
          },
          {
            tagName: "image",
          },
          {
            tagName: "text",
            selector: "label",
          },
        ],
        attrs: {
          body: {
            stroke: "#5F95FF",
            fill: "#5F95FF",
          },
          image: {
            width: 26,
            height: 26,
            refX: 13,
            refY: 16,
          },
          label: {
            refX: 3,
            refY: 2,
            textAnchor: "left",
            textVerticalAnchor: "top",
            fontSize: 12,
            fill: "#fff",
          },
        },
        ports: { ...ports },
      },
      true
    );

    const r1 = graph.createNode({
      shape: "custom-rect",
      label: "开始",
      attrs: {
        body: {
          rx: 20,
          ry: 26,
        },
      },
    });
    const r2 = graph.createNode({
      shape: "custom-rect",
      label: "过程",
    });
    const r3 = graph.createNode({
      shape: "custom-rect",
      attrs: {
        body: {
          rx: 6,
          ry: 6,
        },
      },
      label: "可选过程",
    });
    const r4 = graph.createNode({
      shape: "custom-polygon",
      attrs: {
        body: {
          refPoints: "0,10 10,0 20,10 10,20",
        },
      },
      label: "决策",
    });
    const r5 = graph.createNode({
      shape: "custom-polygon",
      attrs: {
        body: {
          refPoints: "10,0 40,0 30,20 0,20",
        },
      },
      label: "数据",
    });
    const r6 = graph.createNode({
      shape: "custom-circle",
      label: "连接",
    });

    const r7 = graph.createNode({
      shape: "custom-rect",
      label: " ",
    });

    stencil.load([r1, r2, r3, r4, r5, r6, r7], "group1");

    const imageShapes = [
      {
        label: "Client",
        // image:
        //   "https://gw.alipayobjects.com/zos/bmw-prod/687b6cb9-4b97-42a6-96d0-34b3099133ac.svg",
        image: img102,
      },
      {
        label: "Http",
        // image:
        //   "https://gw.alipayobjects.com/zos/bmw-prod/dc1ced06-417d-466f-927b-b4a4d3265791.svg",
        image: imgavatar,
      },
      {
        label: "Api",
        image:
          "https://gw.alipayobjects.com/zos/bmw-prod/c55d7ae1-8d20-4585-bd8f-ca23653a4489.svg",
      },
      {
        label: "Sql",
        image:
          "https://gw.alipayobjects.com/zos/bmw-prod/6eb71764-18ed-4149-b868-53ad1542c405.svg",
      },
      {
        label: "Clound",
        image:
          "https://gw.alipayobjects.com/zos/bmw-prod/c36fe7cb-dc24-4854-aeb5-88d8dc36d52e.svg",
      },
      {
        label: "Mq",
        image:
          "https://gw.alipayobjects.com/zos/bmw-prod/2010ac9f-40e7-49d4-8c4a-4fcf2f83033b.svg",
      },
    ];
    const imageNodes = imageShapes.map((item) =>
      graph.createNode({
        shape: "custom-image",
        label: item.label,
        attrs: {
          image: {
            "xlink:href": item.image,
          },
        },
      })
    );
    stencil.load(imageNodes, "group2");
    // #endregion
  },
  computed: {},
  watch: {
    selectedCell(val){
      if(val==null){
        this.onClose()
      }
    }
  },
  methods: {
    exportJson() {
      if (m_graph) {
        const json = m_graph.toJSON();
        console.log(json, 609);
      }
    },
    handleChange(value) {
      console.log(`selected ${value}`, 695);
    },
    showDrawer() {
      this.rightBoxVisible = true;
    },
    onClose() {
      this.rightBoxVisible = false;
      this.selectedCell = null;
    },
    undo() {
      console.log(m_graph.canUndo(), 779);
      if (m_graph) {
        this.isundo = m_graph.canUndo();
        m_graph.canUndo() && m_graph.undo();
      }
    },
    redo() {
      if (m_graph) {
        this.isredo = m_graph.canRedo();
        m_graph.canRedo() && m_graph.redo();
      }
    },
    lineTypeChange(val) {
      console.log(val, 815);
      this.line.type = val;
      let strokeDasharray = null;
      if (val == "dashed") {
        strokeDasharray = this.line.strokeDasharray;
      } else {
        this.line.animation = false;
        this.lineAnimationChange(false);
      }
      let cell = this.selectedCell;
      if (!cell) return;
      cell.setAttrs({
        line: {
          strokeDasharray: strokeDasharray,
        },
      });

      // line: {
      // stroke: "#A2B1C3",
      // stroke: "blue",
      // strokeWidth: 4,
      // strokeDasharray: 0,
      // targetMarker: {
      //   name: "block",
      //   width: 12,
      //   height: 8,
      // },
      // targetMarker: "classic",
      // style: {
      //   animation:
      //     this.line.animation == "center"
      //       ? "ant-line 30s infinite linear"
      //       : "",
      // },
    },
    lineWidthChange(val) {
      console.log(this.line.width, val, 821);
      let cell = this.selectedCell;
      if (!cell) return;
      cell.setAttrs({
        line: {
          strokeWidth: this.line.width,
        },
      });
    },
    lineColorChange(val) {
      console.log(this.line.stroke, val, 881);
      let cell = this.selectedCell;
      if (!cell) return;
      cell.setAttrs({
        line: {
          stroke: this.line.stroke,
        },
      });
    },
    lineAnimationChange(val) {
      console.log(val, 893);
      let cell = this.selectedCell;
      if (!cell) return;
      cell.setAttrs({
        line: {
          style: {
            animation: this.line.animation
              ? "ant-line 30s infinite linear"
              : "",
          },
        },
      });
    },
  },
};
</script>
<style scoped>

.operationbtn{
  margin-right:5px ;
}
</style>



<style>
@keyframes ant-line {
  to {
    stroke-dashoffset: -1000;
  }
}
</style>